from xml.etree.ElementTree import Element, SubElement, tostring
from xml.dom import minidom
from phoebusgen.widget._property import Property


class Widget(object):
    def __init__(self, w_type, name, x_pos, y_pos, width, height):
        self.root = Element('widget', type=w_type, version='2.0.0')
        name_child = SubElement(self.root, 'name')
        name_child.text = name

        x_child = SubElement(self.root, 'x')
        x_child.text = str(x_pos)

        y_child = SubElement(self.root, 'y')
        y_child.text = str(y_pos)

        width_child = SubElement(self.root, 'width')
        width_child.text = str(width)

        height_child = SubElement(self.root, 'height')
        height_child.text = str(height)

        self.prop_factory = Property(self.root)

    def visible(self, visible):
        child = SubElement(self.root, 'visible')
        child.text = str(visible)

    def class_name(self, name):
        pass

    def rule(self, rule):
        pass

    def scripts(self, script):
        pass

    def tool_tip(self, tool_tip):
        child = SubElement(self.root, 'tooltip')
        child.text = tool_tip

    def find_element(self, tag):
        element = self.root.find(tag)
        return element

    def remove_element(self, tag):
        element = self.find_element(tag)
        if element is not None:
            self.root.remove(element)

    def get_element_value(self, tag):
        return self.find_element(tag).text

    # From: https://pymotw.com/3/xml.etree.ElementTree/create.html
    def prettify(self, elem):
        """Return a pretty-printed XML string for the Element.
        """
        rough_string = tostring(elem, 'utf-8')
        reparse_xml = minidom.parseString(rough_string)
        return reparse_xml.toprettyxml(indent="  ", newl="\n")

    def __str__(self):
        return self.prettify(self.root)


if __name__ == '__main__':
    pass



class PVName(object):
    def pv_name(self, name):
        self.prop_factory.add_pv_name(name)


class Font(object):
    def predefined_font(self, name):
        self.prop_factory.add_font(None, None, None, name)

    def font(self, family=None, style=None, size=None):
        self.prop_factory.add_font(family, style, size, None)


class ForegroundColor(object):
    def predefined_foreground_color(self, name):
        self.prop_factory.add_foreground_color(name, None, None, None, None)

    def foreground_color(self, red, green, blue, alpha=255):
        self.prop_factory.add_foreground_color(None, red, green, blue, alpha)


class BackgroundColor(object):
    def predefined_background_color(self, name):
        self.prop_factory.add_background_color(name, None, None, None, None)

    def background_color(self, red, green, blue, alpha=255):
        self.prop_factory.add_background_color(None, red, green, blue, alpha)


class Transparent(object):
    def transparent(self, transparent=False):
        self.prop_factory.add_transparent(transparent)


class Format(object):
    def format(self, format):
        self.prop_factory.add_format(format)


class Precision(object):
    def precision(self, val):
        self.prop_factory.add_precision(val)

class ShowUnits(object):
    def show_units(self, show=True):
        self.prop_factory.add_show_units(show)


class HorizontalAlignment(object):
    def horizontal_alignment(self, val):
        self.prop_factory.add_horizontal_alignment(val)


class VerticalAlignment(object):
    def vertical_alignment(self, val):
        self.prop_factory.add_vertical_alignment(val)


class WrapWords(object):
    def wrap_words(self, wrap=True):
        self.prop_factory.add_wrap_words(wrap)


class Text(object):
    def text(self, text):
        self.prop_factory.add_text(text)


class AutoSize(object):
    def auto_size(self, auto=True):
        self.prop_factory.add_auto_size(auto)


# 0, 90, 180, -90
# text update, label, action button
class RotationStep(object):
    def rotation_step(self, rotation):
        self.prop_factory.add_rotation_step(rotation)


# picture, symbol
class Rotation(object):
    pass


class File(object):
    pass

class Border(object):
    def border_width(self, width):
        self.prop_factory.add_border_width(width)

    def predefined_border_color(self, name):
        self.prop_factory.add_border_color(name, None, None, None, None)

    def border_color(self, red, green, blue, alpha=255):
        self.prop_factory.add_border_color(None, red, green, blue, alpha)


class Macro(object):
    def macro(self, name, val):
        self.prop_factory.add_macro(name, val)


class Bit(object):
    def bit(self, val=-1):
        self.prop_factory.add_bit(val)


class Off(object):
    def predefined_off_color(self, name):
        self.prop_factory.add_off_color(name, None, None, None, None)

    def off_color(self, red, green, blue, alpha=255):
        self.prop_factory.add_off_color(None, red, green, blue, alpha)

    def off_label(self, label):
        self.prop_factory.add_off_label(label)


class On(object):
    def predefined_on_color(self, name):
        self.prop_factory.add_on_color(name, None, None, None, None)

    def on_color(self, red, green, blue, alpha=255):
        self.prop_factory.add_on_color(None, red, green, blue, alpha)

    def on_label(self, label):
        self.prop_factory.add_on_label(label)


# Arc, Ellipse, Polygon, Polyline, Rectangle, LED, LED Multi State
class LineColor(object):
    def predefined_line_color(self, name):
        self.prop_factory.add_line_color(name, None, None, None, None)

    def line_color(self, red, green, blue, alpha=255):
        self.prop_factory.add_line_color(None, red, green, blue, alpha)


# Arc, Ellipse, Polygon, Polyline, Rectangle
class LineWidth(object):
    def line_width(self, width):
        self.prop_factory.add_line_width(width)


# Rectangle
class Corner(object):
    def corner_width(self, width):
        self.prop_factory.add_corner_width(width)

    def corner_height(self, height):
        self.prop_factory.add_corner_height(height)


class Square(object):
    def square(self, val):
        self.prop_factory.add_square(val)


class LabelsFromPV(object):
    def labels_from_pv(self, val):
        self.prop_factory.add_labels_from_pv(val)


class AlarmBorder(object):
    def alarm_border(self, val):
        self.prop_factory.add_alarm_border(val)


class Enabled(object):
    def enabled(self, val):
        self.prop_factory.add_enabled(val)


class Confirmation(object):
    def confirmation_dialog(self, message, password=None):
        self.prop_factory.add_confirmation_dialog(True)
        self.prop_factory.add_confirmation_message(message)
        if password is not None:
            self.prop_factory.add_password(password)


class MultiLine(object):
    def multi_line(self, val):
        self.prop_factory.add_multi_line(val)


class Angle(object):
    def angle_start(self, val):
        self.prop_factory.add_angle_start(val)

    def angle_size(self, val):
        self.prop_factory.add_angle_size(val)


from phoebusgen.widget._widget import Widget
from phoebusgen.widget._property_stubs import *

# Displays

class Arc(Widget, Macro, Angle, LineWidth, LineColor, BackgroundColor, Transparent):
    """ Arc widget, req: none """
    def __init__(self, name, x, y, width, height):
        Widget.__init__(self, 'arc', name, x, y, width, height)


class Ellipse(Widget, Macro, LineWidth, LineColor, BackgroundColor, Transparent):
    """ Ellipse widget, req: none """
    def __init__(self, name, x, y, width, height):
        Widget.__init__(self, 'ellipse', name, x, y, width, height)


class Rectangle(Widget, Macro, LineWidth, LineColor, BackgroundColor, Transparent, Corner):
    """ Rectangle widget, req: none """
    def __init__(self, name, x, y, width, height):
        Widget.__init__(self, 'rectangle', name, x, y, width, height)


class Label(Widget, Text, Macro, Font, ForegroundColor, BackgroundColor, Transparent, HorizontalAlignment,
            VerticalAlignment, RotationStep, WrapWords, AutoSize, Border):
    """ Label widget, req: text """
    def __init__(self, name, text, x, y, width, height):
        Widget.__init__(self, 'label', name, x, y, width, height)

        self.text(text)

# Monitors

class TextUpdate(Widget, PVName, Font, ForegroundColor, BackgroundColor, Transparent,
                 Format, Precision, ShowUnits, HorizontalAlignment, VerticalAlignment, WrapWords,
                 RotationStep, Border):
    """ Text Update widget, req: pv name """
    def __init__(self, name, pv_name, x, y, width, height):
        Widget.__init__(self, 'textupdate', name, x, y, width, height)
        self.pv_name(pv_name)

# Class, Bit, Off Label, Off Color, On Label, On Color, Line Color, Square, Labels from PV,
# Alarm border
## What should be required in init???
class LED(Widget, PVName, Bit, Off, On, Font, ForegroundColor, LineColor,
          Square, LabelsFromPV, AlarmBorder):
    """ LED widget, req: None """
    def __init__(self, name, pv_name, x, y, width, height):
        Widget.__init__(self, 'led', name, x, y, width, height)
        self.pv_name(pv_name)

# Controls
# Actions, Rotation, Enabled, Confirmation Dialog, Confirmation Message, Password

## What should be required in init???
class ActionButton(Widget, PVName, Text, Font, ForegroundColor, BackgroundColor,
                   Transparent, RotationStep, Enabled, Confirmation):
    """ Action button widget, req: text, pv name """
    def __init__(self, name, text, pv_name, x, y, width, height):
        Widget.__init__(self, 'action_button', name, x, y, width, height)
        self.pv_name(pv_name)
        self.text(text)


class TextEntry(Widget, PVName, Font, ForegroundColor, BackgroundColor, Format,
                Precision, ShowUnits, WrapWords, MultiLine, AlarmBorder, Enabled,
                Border):
    """ Text Entry widget, req: pv name """
    def __init__(self, name, pv_name, x, y, width, height):
        Widget.__init__(self, 'textentry', name, x, y, width, height)
        self.pv_name(pv_name)


if __name__ == '__main__':
    pass

